#ifndef _GRAPHEN_H_
#define _GRAPHEN_H_

// Liste mit Beispielgraphen im CSR Format 
#include <iostream>
#include "math.h"
using namespace std;
#include <cstdlib>

void gitter(int n, int m,int* column_indices,double* entries,
              int* row_offsets)
// nxm Gitter als CSR-Format
// Gewichte: 1,2,3,4 li,re,un,ob
{
  int i,j,l,pos,nnz,ke;
  pos=0;
  nnz=0;
  ke=0;
  for (i=0;i<n;i++)
      {
      for (j=0;j<m;j++) 
        {
          l=0; 
	  if (i>0) 
	    {
	      entries[ke]=1;
              column_indices[ke++]=pos-m;
              l++; 
            }
           if (j>0) 
	    {
	      entries[ke]=2;
              column_indices[ke++]=pos-1;
              l++ ;
            } 
           if (j<m-1) 
	    {
	      entries[ke]=3;
              column_indices[ke++]=pos+1;
              l++; 
            } 
           if (i<n-1) 
	    {
	      entries[ke]=4;
              column_indices[ke++]=pos+m;
              l++;
            } 
           row_offsets[pos]=nnz;
           nnz=nnz+l;
           pos++;
	}
      } 
  row_offsets[pos]=nnz; 
}

void Netzwerk_gitter(int n, int m,int* column_indices,
                     double* entries,int* row_offsets)
// nxm Gitter als CSR-Format
// gerichtet von untenlinks nach rechts oben
// Gewichte: (m^2/(m-|i+j-m|+1)
{
  int i,j,l,pos,nnz,ke;
  pos=0;
  nnz=0;
  ke=0;
  for (i=0;i<n;i++)
      {
      for (j=0;j<m;j++) 
        {
          l=0;
          if (i>0) 
	    {
	      entries[ke]=0;
              column_indices[ke++]=pos-m;
              l++; 
            }
           if (j>0) 
	    {
	      entries[ke]=0;
              column_indices[ke++]=pos-1;
              l++ ;
            } 

          if (j<m-1) 
	    {
	      entries[ke]=double((m*m/(2*m-abs(2*i+2*j-2*m+3))));
		//1+double((m-1-2*j)*(m-1-2*j)+(n-1-2*i)*(n-1-2*i));
              column_indices[ke++]=pos+1;
              l++; 
            } 
           if (i<n-1) 
	    {
	      entries[ke]=double((m*m/(2*m-abs(2*i+2*j-2*m+3))));
		//1+double((m-1-2*j)*(m-1-2*j)+(n-1-2*i)*(n-1-2*i));
              column_indices[ke++]=pos+m;
              l++;
            } 
           row_offsets[pos]=nnz;
           nnz=nnz+l;
           pos++;
	}
      } 
  row_offsets[pos]=nnz; 
}

void vollst(int n,int* column_indices,double* entries,
                   int* row_offsets)
// vollst Graph als CSR Format
// Gewichte i+j
{
  int i,j,nnz,ke;
  nnz=0;
  ke=0;
  for (i=0;i<n;i++)
    {
      for (j=0;j<n;j++) 
      {
	if (j!=i)
          {
          entries[ke]=i+j+1;
          column_indices[ke++]=j;
          }
      }
     row_offsets[i]=nnz;
     nnz=nnz+n-1;
    }
   row_offsets[n]=nnz;
}

void Netzwerk_vollst(int n,int* column_indices,
           double* entries,int* row_offsets)
// vollst Graph als Netzwerk, d.h. e=(i,j) falls i<j
// Gewichte sind zufaellig (am Anfang bestimmt)
{
  int i,j,nnz,ke;
  nnz=0;
  ke=0;
  for (i=0;i<n;i++)
    {
      for (j=0;j<n;j++) 
      {
	if (j>i)
          {
          column_indices[ke++]=j;
          }
        if (j<i)
          {
          entries[ke]=0;
          column_indices[ke++]=j;
          }
      }
     row_offsets[i]=nnz;
     nnz=nnz+n-1;
    }
   row_offsets[n]=nnz;
}

void baum(int k,int* column_indices,double* entries,
                                  int* row_offsets)
// BinaerBaum mit 2^k-1 Knoten als CSR-Format
// Gewichte: -1 nach oben; 
//            1 nach links
//            2 nach rechts 
{
  int i,j,n,l,m,nnz,ke;
  n=pow(2,k)-1;
  nnz=0;
  ke=0;

  entries[ke]=1;
  column_indices[ke++]=1;  
  entries[ke]=2;
  column_indices[ke++]=2;  
  row_offsets[0]=0;
  nnz=nnz+2;
  i=0;
  for (l=1;l<k-1;l++)
    {
    m=pow(2,l);
    for (j=0;j<m;j++) 
      {
        cout << j <<" "<< m <<endl;
	i++;
        entries[ke]=-1;
	column_indices[ke++]=(i-1)/2;
        entries[ke]=1;
	column_indices[ke++]=2*i+1;
        entries[ke]=2;
	column_indices[ke++]=2*i+2;
        row_offsets[i]=nnz;
        nnz=nnz+3;
      }
    }
   m=pow(2,k-1);
   for (j=0;j<m;j++) 
      {
	i++;
        entries[ke]=-1;
	column_indices[ke++]=(i-1)/2;
        
        row_offsets[i]=nnz;
        nnz=nnz+1;
      }     
    row_offsets[n]=nnz;
    
}
#endif
